
/*
 * MGL graphics library
 * Copyright (C) 1999-2000, ing. Marek Gayer

 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.

 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdlib.h>
#include <GL/glut.h>

#include <mgl.h>

extern void mainMakeCheckImage (unsigned char *map, int size, int segs);
extern int mainOption;
extern int mainSpeed;

static unsigned char mipmapImage64 [64*64*3];
static unsigned char mipmapImage32 [32*32*3];
static unsigned char mipmapImage16 [16*16*3];
static unsigned char mipmapImage8 [8*8*3];
static unsigned char mipmapImage4 [4*4*3];
static unsigned char mipmapImage2 [2*2*3];
static unsigned char mipmapImage1 [1*1*3];

static mglImage *checkImage, *mipmapImage;

static mglTexture *checkTex, *checkTexNearest, *checkTexLinear, *checkTex1, *checkTex2, *checkTex3, *checkTex4;
static mglTexture *mipmapTex, *curTex;


static void makemipmapImages(void)
{
   int i, j;

   for (i = 0; i < 64; i++) {
      for (j = 0; j < 64; j++) {
         mipmapImage64[3*j+3*64*i+0] = 0;
         mipmapImage64[3*j+3*64*i+1] = 255;
         mipmapImage64[3*j+3*64*i+2] = 255;
      }
   }
   for (i = 0; i < 32; i++) {
      for (j = 0; j < 32; j++) {
         mipmapImage32[3*j+3*32*i+0] = 255;
         mipmapImage32[3*j+3*32*i+1] = 0;
         mipmapImage32[3*j+3*32*i+2] = 255;
      }
   }
   for (i = 0; i < 16; i++) {
      for (j = 0; j < 16; j++) {
         mipmapImage16[3*j+3*16*i+0] = 255;
         mipmapImage16[3*j+3*16*i+1] = 255;
         mipmapImage16[3*j+3*16*i+2] = 0;
      }
   }
   for (i = 0; i < 8; i++) {
      for (j = 0; j < 8; j++) {
         mipmapImage8[3*j+3*8*i+0] = 255;
         mipmapImage8[3*j+3*8*i+1] = 0;
         mipmapImage8[3*j+3*8*i+2] = 0;
      }
   }
   for (i = 0; i < 4; i++) {
      for (j = 0; j < 4; j++) {
         mipmapImage4[3*j+3*4*i+0] = 0;
         mipmapImage4[3*j+3*4*i+1] = 255;
         mipmapImage4[3*j+3*4*i+2] = 0;
      }
   }
   for (i = 0; i < 2; i++) {
      for (j = 0; j < 2; j++) {
         mipmapImage2[3*j+3*2*i+0] = 0;
         mipmapImage2[3*j+3*2*i+1] = 0;
         mipmapImage2[3*j+3*2*i+2] = 255;
      }
   }
   mipmapImage1[0] = 255;
   mipmapImage1[1] = 255;
   mipmapImage1[2] = 255;
}

int mipmapOption;

static void setMipmapOption (void)
{
  if (mipmapOption == mainOption)
    return;
  mipmapOption = mainOption;

  switch (mainOption)
  {
    case 1:
      curTex = mipmapTex;
      mglSetWindowTitle ("Demonstrace principu mipmappingu");
      break;
    case 2:
      curTex = checkTexNearest;
      mglSetWindowTitle ("N - Bez mipmappingu a bez filtrovni");
      break;
    case 3:
      curTex = checkTexLinear;
      mglSetWindowTitle ("L - Bez mipmappingu s bilinernm filtrovnm");
      break;
    case 4:
      curTex = checkTex1;
      mglSetWindowTitle ("NN - Mipmapping bez fitrovn");
      break;
    case 5:
      curTex = checkTex2;
      mglSetWindowTitle ("LMN - Mipmapping s bilinernm filtrovnm");
      break;
    case 6:
      curTex = checkTex3;
      mglSetWindowTitle ("NML - Mipmapping s trilinernm filtrovnm");
      break;
    case 7:
      curTex = checkTex4;
      mglSetWindowTitle ("LML - Mipmapping s trilinernm filtrovnm");
      break;
   }
}

void mipmapDisplay (void)
{
  setMipmapOption ();

  mglBegin ();

  glDisable(GL_DEPTH_TEST);
  glDisable(GL_CULL_FACE);

  if (mainSpeed > 5)
    glHint (GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
  else
    glHint (GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

  glEnable(GL_DEPTH_TEST);
  glViewport(0, 0, mglWindowWidth, mglWindowHeight);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

  gluPerspective(40.0, (GLfloat)mglWindowWidth/(GLfloat)mglWindowHeight, 1, 30.0);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  gluLookAt(0, 1, 3, 0, 0, 0, 0, 1, 0);

  glClearColor (0.6f, 0.8f, 1, 1);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glPolygonMode (GL_BACK, GL_FILL);
  glDisable (GL_LIGHTING);
  glShadeModel (GL_SMOOTH);

  mglChangeTexture (curTex);

  glEnable (GL_TEXTURE_2D);
  glBegin(GL_POLYGON);

  glColor4f (1, 1, 1, 1);
  glTexCoord2f(0.0, 300.0); glVertex3f(-2, 0, -300); // from point
  glTexCoord2f (8.0, 300.0); glVertex3f(2, 0, -300);

  glTexCoord2f(8.0, 0.0); glVertex3f(2, 0, 7);
  glTexCoord2f(0, 0); glVertex3f(-2, 0, 7);
  glEnd ();

  mglEnd ();
}


int mipmapInit (void)
{
  unsigned char *mipmaps [] = {mipmapImage32, mipmapImage16, mipmapImage8,
    mipmapImage4, mipmapImage2, mipmapImage1, NULL};

  if (!mipmapImage)
  {
    mglNote ("Initializing mipmaps ...\n");
    mipmapImage = mglCreateImage (64, 64, 3);
    _free (mipmapImage->data);
    makemipmapImages();
    mipmapImage->data = mipmapImage64;
    mipmapImage->mipmap = mipmaps;
    mipmapImage->type = GL_RGB;
    mipmapTex = mglMakeTexture (mipmapImage, GL_NEAREST_MIPMAP_NEAREST);

    checkImage = mglCreateImage (64, 64, 1);
    mainMakeCheckImage (checkImage->data, 64, 32);
    checkImage->type = GL_LUMINANCE;
    checkTexNearest = mglMakeTexture (checkImage, GL_NEAREST);
    checkTexLinear = mglMakeTexture (checkImage, GL_LINEAR);

    checkTex1 = mglMakeTexture (checkImage, GL_NEAREST_MIPMAP_NEAREST);
    checkTex2 = mglMakeTexture (checkImage, GL_LINEAR_MIPMAP_NEAREST);
    checkTex3 = mglMakeTexture (checkImage, GL_NEAREST_MIPMAP_LINEAR);
    checkTex4 = mglMakeTexture (checkImage, GL_LINEAR_MIPMAP_LINEAR);
    mglNote ("Initializing mipmaps is done.\n");

    curTex = mipmapTex;
  }

//  mainOption = 0;
  mipmapOption = -1;

//  mglSetKeyboardHandler (keyboard);
//  mglSetDisplayHandler (display);

//  mglRun ();

  return 0;
}
