
#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include <windows.h>
#include <winuser.h>

#include <gl/gl.h>
#include <gl/glu.h>

#include "mgl.h"

#define APP_NAME "MGL using WGL"

static int done;

static int leftFirst;

void display (void) 
{
  mglColor leftColor = {1.0, 1.0, 0.0, 0.75};
  mglColor rightColor = {0.0, 1.0, 1.0, 0.75};

  mglVirtualWidth = 1000;
  mglVirtualHeight = 1000;

  mglBegin ();

  mglClearScreen (mglBlack);

 if (leftFirst)
 {
   mglSetTriangleColors (rightColor);
   mglDrawFilledTriangle (100, 100, 100, 900, 700, 500);
   mglSetTriangleColors (leftColor);
   mglDrawFilledTriangle (300, 500, 900, 900, 900, 100);
 }
 else 
 {
   mglSetTriangleColors (leftColor);
   mglDrawFilledTriangle (300, 500, 900, 900, 900, 100);
   mglSetTriangleColors (rightColor);
   mglDrawFilledTriangle (100, 100, 100, 900, 700, 500);
 }

  mglEnd ();

  glFlush();
}


HGLRC setupGL (HDC hDC) 
{
  int pixelFormat;
  HGLRC rv = 0;
  PIXELFORMATDESCRIPTOR pfd = {
    sizeof(PIXELFORMATDESCRIPTOR),  // size of this pfd
    1,                              // version number
    PFD_DRAW_TO_WINDOW              // support window
    |  PFD_SUPPORT_OPENGL           // support OpenGL
    |  PFD_DOUBLEBUFFER ,           // double buffered
    PFD_TYPE_RGBA,                  // RGBA type
    16,                             // 16-bit color depth
    0, 0, 0, 0, 0, 0,               // color bits ignored
    0,                              // no alpha buffer
    0,                              // shift bit ignored
    0,                              // no accumulation buffer
    0, 0, 0, 0,                     // accum bits ignored
    16,                             // 16-bit z-buffer      
    0,                              // no stencil buffer
    0,                              // no auxiliary buffer
    PFD_MAIN_PLANE,                 // main layer
    0,                              // reserved
    0, 0, 0                         // layer masks ignored
  };

  pixelFormat = ChoosePixelFormat (hDC, &pfd);
  if (pixelFormat)
  {
    if (SetPixelFormat (hDC, pixelFormat, &pfd)) 
    {
      rv = wglCreateContext (hDC);
      if (rv) 
      {
        if (!wglMakeCurrent (hDC, rv)) 
        {
          wglDeleteContext (rv);
          rv = 0;
        }
      }
    }
  }
  return rv;
}

LONG WINAPI MainWndProc (HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  LONG lRet = 1;

  switch (uMsg) 
  {
    case WM_SIZE:
      break;
    case WM_CLOSE:
      DestroyWindow (hWnd);
      break;
    case WM_DESTROY:
      PostQuitMessage (0);
	  break;
    case WM_QUIT:
      done = TRUE;
      break;
    case WM_KEYDOWN:
		  switch (wParam)
		  {
  		  case VK_INSERT:
	  	  case VK_ESCAPE:
          DestroyWindow (hWnd);
          break;
		    case VK_UP:
		    case VK_DOWN:
			    break;
		  }
		  break;
    case WM_CHAR:
      if (tolower (wParam) == 't')
        leftFirst = !leftFirst;
	    break;
    default:
      lRet = DefWindowProc (hWnd, uMsg, wParam, lParam);
      break;
  }
  return lRet;
}

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, 
                    int nCmdShow) 
{
	WNDCLASS wc;
  MSG msg;
  HGLRC glRC;
  HWND hwnd;
  HDC hDC;

  int WinWidth = 208;
  int WinHeight= 224;

  wc.style         = 0;
  wc.lpfnWndProc   = (WNDPROC)MainWndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = LoadIcon (hInstance, "GLUT_ICON");
  wc.hCursor       = LoadCursor( NULL, IDC_ARROW );
  wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
  wc.lpszMenuName  = APP_NAME;
  wc.lpszClassName = APP_NAME;

  if (!RegisterClass (&wc))
  {
    MessageBox( 0, "Unable to create register window class", APP_NAME, MB_ICONERROR);
  }

  hwnd = CreateWindow (APP_NAME, APP_NAME,
			                 WS_CLIPSIBLINGS | WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN,
                       100, 100, WinWidth, WinHeight, 
                       NULL, NULL, hInstance, NULL);
    
  if (!hwnd)
    MessageBox( 0, "Unable to create window", APP_NAME, MB_ICONERROR);

	hDC  = GetDC( hwnd );
	glRC = setupGL (hDC);
	if (!glRC)
  {
		MessageBox( 0, "Unable to create OpenGL rendering context.", APP_NAME, MB_ICONERROR);
		DestroyWindow( hwnd );
	}
  
  ShowWindow( hwnd, SW_SHOWDEFAULT );
	UpdateWindow(hwnd);
	SetFocus (hwnd);

//  init ();

  while (!done) 
  {
    while (PeekMessage (&msg, 0, 0, 0, PM_REMOVE)) 
    {
		  TranslateMessage (&msg);
		  DispatchMessage (&msg);
		  if (msg.message == WM_QUIT)
			  done = 1;
    }
    display (); 

	  SwapBuffers (hDC);
  }

	if (hDC)
		ReleaseDC (hwnd, hDC);

	wglDeleteContext (glRC);

	return msg.wParam;
}
